/**
 *	netdef.h  --  header file for tcp/ip network programs
 *		Copyright (C) Kazuro Furukawa, 1988-2003
 *		Kazuro Furukawa, KEK, Japan <kazuro.furukawa@kek.jp>
 *
 *	This file includes platform dependent network header files.
 *	It has two parts.  The first part is used by ANSI compiler 
 *	and is modified actively.  The second part is used by (old)
 *	K&R compiler and almost fixed at 1995 version.
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.Furukawa  Mar.7.1990.  separate header part, 1st version (netdef.h)
 *	Modifications
 *	K.Furukawa  Apr.23.1990. for fujitsu/a60/sxa
 *	K.F         Jul.10.1990. for sg/iris/irix
 *	K.F         Jul.16.1990. for os9/68k
 *	K.F         Aug.7.1990.  time definitions
 *	K.F         Aug.9.1990.  for NeXT
 *	K.F         Oct.17.1990. for ibm/aix
 *	K.F         Mar.7.1991.  signal reinstallation
 *	K.F         Mar.30.1991. for msdos/pctcp(by ftp software inc.)
 *	K.F         Apr.5.1991.  for apple/macintosh/aux
 *	K.F         Mar.5.1992.  for lynx-os, using non_bsd macro
 *	K.F         Mar.10.1992. for vms/multinet (as well as wintcp)
 *	K.F         Jun.17.1992. bug of accept on msdos and osk
 *	K.F         Jan.7.1993.  for vms/ucx (as well as wintcp/multinet)
 *	K.F         Apr.18.1993. for alpha/osf1
 *	K.F         Jun.12.1993. noselect, etc., use sys/time instead of time
 *	K.F         Feb.11.1994. select support for hpux and osk
 *	K.F         Jul.18.1994. LInt definition
 *	K.F         Feb.21.1996. time functions, memory functions, ansi
 *	K.F         Dec.22.1997. using sigaction() & waitpid() SIGACTION
 *	K.F         Feb.9.1998.  for windows32 with gnu-win32
 *	K.F         Jan.9.1999.  for windows32 gnu-win32, opennt, uwin
 *	K.F         Apr.22.1999. include timeb.h for ftime again
 *	K.F         May.11.1999. for vxWorks, using nonetdb
 *	K.F         Apr.1.2000.  for windows32 winsock
 *	K.F         Apr.22.2003. for apple darwin
 *	K.F         Aug.17.2003. add unistd.h and stdlib.h, add freebsd
 *	K.F         Jul.28.2006. minor fix for solaris8
 *
 *	This contains system dependent part of the tcp/ip network
 *	programs.  Especially header files for tcp/ip definitions are
 *	scattered in the system directories.  
 *	The default definition is the BSD style, which is defined at
 *	the end of this file.  If the BSD style cannot be used, 
 *	appropriate definitions are chosen. 
 *	One of the macro vms, sony, __sun, __ultrix, NeXT, __osf__, 
 *	__hpux, SXA, sgi, _AIX, __bsdi__, __linux__, __FreeBSD__, 
 *	ebcdic, _AUX_SOURCE, Lynx, __CYGWIN32__, __OPENNT, _OSK, 
 *	MSDOS, _WIN32
 *	is defined in the language system of each platform.  
 *	If you use MULTINET or UCX on vms, you should define one of 
 *	them explicitly. (WINTMP is the default on vms.)
 *
 * @(#)netdef.h v2.21.10 tcp/ip header file, K.Furukawa, KEK, Nov.1988-Jul.2006
 *
 *	SINGLESERVER :	no unix fork facility
 *	NON_BSD :	no bsd style header files
 *	NOLINGERTIME :	no linger structure
 *	SIGACTION:      use sigaction() and waitpid() instead of signal/wait
 *	REINSTALLSIGNAL: signal reinstallation needed on systemV
 *	USEFTIME :	use ftime() instead of gettimeofday() or time()
 *	USETIME :	use time() instead of gettimeofday() or ftime()
 *	NOSELECT :	no select()
 *	BADACCEPT :	accept() bug in return value
 *	NEEDTIMEVAL :	need timeval structure definition at the end
 *	NEEDFDMACRO :	need fd_xxx macros at the end
 *	DONTDEFLINT :	users don't need LInt definition
 *	NONEEDTIMEH :	no need for time.h
 *	LITTLE_END :	little endien
 *	NONETDB :	no netdb routines like get/setxxx()
**/

#ifdef __STDC__				/* for ansi C */

#ifndef __NETDEF_H
#define __NETDEF_H

/***************************************/

#ifdef vms				/* for dec/vms */
#define NOSELECT 1			/* no select() call */
#define USEFTIME 1			/* use ftime insteadof gettimeofday */
#define LITTLE_END 1			/* little endian machine */
#ifndef MULTINET			/* not multinet */
#ifndef UCX				/* not ucx */
#define NON_BSD 1			/* dec/vms-win/tcp non-BSD system */
#endif /* UCX */
#endif /* MULTINET */
#ifdef NON_BSD				/* for dec/vms-win/tcp */
#include stdio				/* standard buffered i/o routines */
#include time				/* time definitions */
#include <sys/types2.h>			/* general data type */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#define SINGLESERVER 1			/* no multiple server */
#else /* NON_BSD */			/* for dec/vms-multinet or ucx */
#define SINGLESERVER 1			/* no multiple server */
#ifdef MULTINET
#define netclose(d) socket_close(d)	/* close function */
#endif /* MULTINET */
#ifdef UCX
#ifndef getservbyname
#define getservbyname(a, b) zerozero()	/* ucx 2.0/3.0 have no getservbyname */
/* void *zerozero()  { return 0; } */
#endif /* getservbyname */
#define netclose(d) close(d)		/* close function */
#endif /* UCX */
					/* using BSD definitions at the end */
#endif /* NON_BSD */
#endif /* vms */

#ifdef sony				/* for sony/news */
#define LITTLE_END 1			/* little endian machine */
					/* using BSD definitions at the end */
#endif /* sony */

#ifdef __sun				/* for smi/sun3 and sun4 and sun5 */
#define NONEEDTIMEH 1			/* time.h,sys/time.h sunos4 conflict */
/* #define SIGACTION 1 */		/* use sigaction() and waitpid() */
					/* using BSD definitions at the end */
#endif /* __sun */

#ifdef __ultrix				/* for dec/ultrix */
#define SIGACTION 1			/* use sigaction() and waitpid() */
#define LITTLE_END 1			/* little endian machine */
					/* using BSD definitions at the end */
#endif /* __ultrix */

#ifdef NeXT				/* for NeXT */
/* it does not work with cc -ansi, use cc instead */
#define SIGACTION 1			/* use sigaction() and waitpid() */
					/* using BSD definitions at the end */
#endif /* NeXT */

#ifdef __osf__				/* for dec/alpha/osf1 */
					/* using BSD definitions at the end */
#define LINT32				/* int is 32bit and long is 64bit */
#define LITTLE_END 1			/* little endian machine */
#define SIGACTION 1			/* use sigaction() and waitpid() */
#endif /* __osf__ */

#ifdef __hpux				/* for hpux */
#ifndef _HPUX_SOURCE
#define _HPUX_SOURCE			/* for socket.h, etc. */
#endif
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines, hpux 8.0- */
#include <netdb.h>			/* network get(set)xxx routines */
#include <sys/wait.h>			/* waitpid() */
/* #define NOSELECT 1 */		/* no bsd select() call */
#define NOLINGERTIME 1			/* no linger structure defined */
/* #define SIGACTION 1 */		/* use sigaction() and waitpid() */
#undef SIGACTION 			/* maybe better to use it, but ... */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define SIGNAL_CAST (void (*)(__harg))	/* signal handler declaration */
#define NON_BSD 1			/* non-BSD system */
#endif /* __hpux */

#ifdef SXA				/* for Fujitsu/A60/SXA */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <net/in.h>			/* another network structures */
#include <net/inet.h>			/* inet_xxx routines */
#include <net/netdb.h>			/* network get(set)xxx routines */
#define NOSELECT 1			/* no select() call ?? */
#define NOLINGERTIME 1			/* no linger structure defined */
#define USETIME 1			/* use time instead of gettimeofday */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif /* SXA */

#ifdef sgi				/* for sg/iris/irix */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <bsd/sys/types.h>		/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <bsd/sys/socket.h>		/* network structures */
#include <bsd/netinet/in.h>		/* another network structures */
#include <bsd/arpa/inet.h>		/* inet_xxx routines */
#include <bsd/netdb.h>			/* network get(set)xxx routines */
#define NOSELECT 1			/* no select() call ?? */
#define USEFTIME 1			/* use ftime instead of gettimeofday */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define SIGNAL_CAST (void (*)())	/* signal handler declaration */
#define NON_BSD 1			/* non-BSD system */
#endif /* sgi */

#ifdef _AIX				/* for ibm/aix */
					/* using BSD definitions at the end */
#include <sys/select.h>			/* for select() call */
/* #define REINSTALLSIGNAL 1 */		/* signal reinstallation */
					/*  not nessesary on aix 3.2 / lator */
#define SIGNAL_CAST (void (*)())	/* signal handler declaration */
#endif /* _AIX */

#ifdef __bsdi__				/* for bsdi/bsdos */
					/* using BSD definitions at the end */
#define SIGCLD SIGCHLD			/* old signal name */
#define LITTLE_END 1			/* little endian machine */
#endif /* __bsdi__ */

#ifdef __linux__			/* for linux */
					/* using BSD definitions at the end */
#define LITTLE_END 1			/* little endian machine */
#include <unistd.h>			/* standard unix system calls */
#endif /* __linux__ */

#ifdef __FreeBSD__			/* for freebsd */
					/* using BSD definitions at the end */
#define LITTLE_END 1			/* little endian machine */
/* #include <unistd.h> */		/* standard unix system calls */
#endif /* __FreeBSD__ */

#ifdef ebcdic				/* for mitsubishi/umx */
#define  TCP_IP  1
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/time.h>			/* time definitions for select */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <sys/net/in.h>			/* another network structures */
/* #include <net/inet.h> */
unsigned long inet_addr();		/* inet_xxx routine */
char *inet_ntoa();			/* inet_xxx routine */
#include <net/netdb.h>			/* network get(set)xxx routines */
/* #define NOSELECT 1 */		/* no select() call ?? */
#define USETIME 1			/* use time instead of gettimeofday */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif /* ebcdic */

#ifdef _AUX_SOURCE			/* for apple/macintosh/aux */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <netdb.h>			/* network get(set)xxx routines */
#define NOSELECT 1			/* no select() call ?? */
#define USETIME 1			/* use time instead of gettimeofday */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif /* _AUX_SOURCE */

#ifdef Lynx				/* for Lynx-OS */
					/* using BSD definitions at the end */
#endif /* Lynx */

#ifdef __APPLE__			/* for Darwin */
					/* using BSD definitions at the end */
#endif /* __APPLE__ */

#ifdef __CYGWIN32__			/* for Windows/Cygwin32 */
					/* using BSD definitions at the end */
#endif /* __CYGWIN32__ */

#ifdef __OPENNT				/* for Windows/Interix/OpenNT */
					/* using BSD definitions at the end */
struct timezone {			/* it's not used on OpenNT, but ... */
    int  tz_minuteswest;		/* of Greenwich */
    int  tz_dsttime;			/* type of dst correction to apply */
};
#define O_NDELAY O_NONBLOCK
#define setpgrp() setpgid()		/* actually setpgid should be used */
#endif /* __OPENNT */


#if (defined(_WIN32) && !defined(__CYGWIN32__) && !defined(__OPENNT))
					/* for Windows/Winsock */
/* #ifdef __BORLANDC__ */		/* also for Windows/Borland-C */
					/* we should use winsock2 soon */
#define WINSOCK 1
#include <stdio.h>			/* standard buffered i/o routines */
#include <stdlib.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <winsock.h>
#include <sys/types.h>			/* general data type */
#include <sys/timeb.h>			/* time definitions for ftime */
#include <signal.h>			/* signal names */
#include <errno.h>			/* for errno */
#ifndef NONEEDTIMEH
#include <time.h>			/* time definitions such as ctime */
#endif /* NONEEDTIMEH */
#ifdef SIGACTION
#include <sys/wait.h>			/* waitpid() */
#endif /* SIGACTION */
#ifndef SIGCLD
#define SIGCLD (SIGCHLD)		/* for bsd (cygwin32) */
#endif /* SIGCLD */
#define SINGLESERVER 1			/* no multiple server */
#define USEFTIME 1			/* use ftime instead of gettimeofday */
#define NOSELECT 1			/* no select() call in 1.3 */
#define NON_BSD 1			/* non-BSD system */
#define LITTLE_END 1			/* little endian machine */
#define sleep(t) s2_null()
#ifndef EWOULDBLOCK
#define EWOULDBLOCK 11
#endif
#ifndef EINPROGRESS
#define EINPROGRESS 119
#endif
#endif /* _WIN32 */

#ifdef _OSK				/* for os9/68k ISP */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <types.h>			/* general data type */
#include <time.h>			/* time definitions */
/* #include <sys/timeb.h> */		/* timeb definitions from cabrales */
/* renamed from inet/ to INET/ since nfs directories are case sensitive */
#include <INET/socket.h>		/* network structures */
#include <INET/in.h>			/* another network structures */
#include <INET/netdb.h>			/* get(set)xxx and inet_xxx routines */
#ifndef perror
extern int errno;
#define perror(str) prerr(0,errno);fprintf(stderr,"%s\n",str)
#endif /* perror */
#define SINGLESERVER 1			/* no multiple server */
#define NOSELECT 1			/* no select() call in 1.3 */
/* select() from cabrales */
#define USEFTIME 1			/* use ftime instead of gettimeofday */
/* ftime() from cabrales */
#define BADACCEPT 1			/* accept() bug */
#define NON_BSD 1			/* non-BSD system */
#define NEEDTIMEVAL 1			/* timeval structure */
#define NEEDTIMEB 1			/* timeb structure */
#define NEEDFDMACRO 1			/* fd_xxx macros */
#define EINTR 4				/* interrupted system call on unix */
#endif /* _OSK */

#ifdef MSDOS				/* for msdos/pctcp(ftp soft. inc.) */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <4bsddefs.h>			/* bsd emulation definitions */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#define SINGLESERVER 1			/* no multiple server */
#define USEFTIME 1			/* use ftime instead of gettimeofday */
#define BADACCEPT 1			/* accept() bug */
#define NON_BSD 1			/* non-BSD system */
#define LINT16				/* int is 16 bit */
#define LITTLE_END 1			/* little endian machine */
#endif /* MSDOS */

#ifdef vxWorks				/* for vxworks systems */
#include <stdio.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <sys/times.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <errno.h>			/* for errno */
/* #include <sys/socket.h> */		/* network structures */
/* #include <netinet/in.h> */		/* another network structures */
/* #include <arpa/inet.h> */		/* inet_xxx routines */
#include <sockLib.h>			/* network structures */
#include <inetLib.h>			/* inet_xxx routines */
#include <timers.h>			/* nanosleep() */
/* #include <netdb.h> */		/* network get(set)xxx routines */
#define NONETDB				/* no netdb routines */
/* #define NOSELECT 1 */		/* no select() call ?? */
#include <selectLib.h>			/* select() routine */
#define NOSOCKFCNTL 1			/* no fcntl() for socket */
#define SINGLESERVER 1			/* no multiple server */
#define USETIME 1			/* use time instead of gettimeofday */
#include <time.h>			/* time definitions such as ctime */
#ifdef SIGACTION
#include <sys/wait.h>			/* waitpid() */
#endif /* SIGACTION */
#ifndef SIGCLD
#define SIGCLD (SIGCHLD)		/* for bsd (cygwin32) */
#endif /* SIGCLD */
#define NON_BSD 1			/* non-BSD system */
#endif /* vxWorks */

/***************************************/

#ifndef NON_BSD				/* for BSD based systems */
#include <stdio.h>			/* standard buffered i/o routines */
#include <stdlib.h>			/* standard c subroutines */
#include <string.h>			/* string/memory, may trouble ... */
#include <sys/types.h>			/* general data type */
#include <sys/time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions for ftime */
/* #include <unistd.h> */		/* standard unix system calls */
/* unistd causes setpgrp trouble on tru64 for backward compatibility */
#include <signal.h>			/* signal names */
#include <errno.h>			/* for errno */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#ifndef NONEEDTIMEH
#include <time.h>			/* time definitions such as ctime */
#endif /* NONEEDTIMEH */
#ifdef SIGACTION
#include <sys/wait.h>			/* waitpid() */
#endif /* SIGACTION */
#ifndef SIGCLD
#define SIGCLD (SIGCHLD)		/* for bsd (cygwin32) */
#endif /* SIGCLD */
#endif /* NON_BSD */

#ifndef SIGNAL_CAST
#define SIGNAL_CAST (void (*)(int))	/* signal handler declaration */
#endif /* SIGNAL_CAST */

#ifdef NEEDMEMORY			/* BSD memory functions, for old BSD */
#define memcpy(d,s,c) bcopy(s,d,c)	/* byte subroutines */
#define memset(d,0,c) bzero(d,c)
#define strchr(s,c) index(s,c)		/* character search */
#define strrchr(s,c) rindex(s,c)
#endif /* NEEDMEMORY */

#ifdef NEEDBSDMEMORY
#define bcopy(s,d,c) memcpy(d,s,c)	/* byte subroutines */
#define bzero(d,c) memset(d,0,c)
#define index(s,c) strchr(s,c)		/* character search */
#define rindex(s,c) strrchr(s,c)
#endif /* NEEDBSDMEMORY */

#ifdef NEEDTIMEVAL			/* timeval structure */
struct timeval {			/*  in sys/time.h on unix */
    long	tv_sec;			/* time in seconds */
    long	tv_usec;		/* remaining time in microseconds */
};
#endif /* NEEDTIMEVAL */

#ifdef NEEDTIMEB			/* timeb structure */
struct timeb {				/*  in sys/timeb.h on unix */
    time_t	time;			/* time in seconds */
    unsigned	short millitm;		/* remaining time in milliseconds */
    short timezone;			/* time zone */
    short dstflag;			/* daylight saving time flag */
};
#endif /* NEEDTIMEB */

#ifndef NOSELECT
#ifdef NEEDFDMACRO			/* fd_xxx macros */
#define NBBY    8
#ifndef FD_SETSIZE
#define FD_SETSIZE	256
#endif
typedef long    fd_mask;
#define NFDBITS (sizeof(fd_mask) * NBBY)
#ifndef howmany
#define howmany(x, y)   (((x)+((y)-1))/(y))
#endif
typedef struct fd_set {
    fd_mask fds_bits[howmany(FD_SETSIZE, NFDBITS)];
} fd_set;
#define FD_SET(n, p)    ((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define FD_CLR(n, p)    ((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define FD_ISSET(n, p)  ((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)      memset((char *)(p), 0, sizeof(*(p)))
typedef int * fd_set_p;
#else /* NEEDFDMACRO */
typedef fd_set * fd_set_p;
#endif /* NEEDFDMACRO */
#endif /* NOSELECT */

#ifndef DONTDEFLINT
#ifdef LINT32
typedef int	LInt32;			/* 32bit integer */
#else  /* LINT32 */
typedef long	LInt32;
#endif  /* LINT32 */
#ifdef LINT16
typedef int	LInt16;			/* 16bit integer */
#else  /* LINT16 */
typedef short	LInt16;
#endif  /* LINT16 */
#endif  /* DONTDEFLINT */

#endif /* __NETDEF_H */

#else  /* __STDC__ */			/* for Kernighan & Ritchie C */

/**
 *	netdef.h  --  header file for tcp/ip network programs
 *		Copyright (C) Kazuro Furukawa, 1988-1994
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.Furukawa  Mar.7.1990.  separate header part, 1st version (netdef.h)
 *	Modifications
 *	K.Furukawa  Apr.23.1990. for fujitsu/a60/sxa
 *	K.F         Jul.10.1990. for sg/iris/irix
 *	K.F         Jul.16.1990. for os9/68k
 *	K.F         Aug.7.1990.  time definitions
 *	K.F         Aug.9.1990.  for NeXT
 *	K.F         Oct.17.1990. for ibm/aix
 *	K.F         Mar.7.1991.  signal reinstallation
 *	K.F         Mar.30.1991. for msdos/pctcp(by ftp software inc.)
 *	K.F         Apr.5.1991.  for apple/macintosh/aux
 *	K.F         Mar.5.1992.  for lynx-os, using non_bsd macro
 *	K.F         Mar.10.1992. for vms/multinet (as well as wintcp)
 *	K.F         Jun.17.1992. bug of accept on msdos and osk
 *	K.F         Jan.7.1993.  for vms/ucx (as well as wintcp/multinet)
 *	K.F         Apr.18.1993. for alpha/osf1
 *	K.F         Jun.12.1993. noselect, etc., use sys/time instead of time
 *	K.F         Feb.11.1994. select support for hpux and osk
 *	K.F         Jul.18.1994. LInt definition
 *
 *	This contains system dependent part of the tcp/ip network
 *	programs.  Especially header files for tcp/ip definitions are
 *	scattered in the system directories.  
 *	The default definition is the BSD style, which is defined at
 *	the end of this file.  If the BSD style cannot be used, 
 *	appropriate definitions are chosen. 
 *	One of the macro vms, sony, sun, ultrix, NeXT, __osf__, hpux, SXA, 
 *	sgi, ebcdic, _AIX, _AUX_SOURCE, Lynx, OSK, MSDOS is defined in the 
 *	language system of each platform.  If you use MULTINET or UCX, 
 *	you should define it explicitly.
 *
 * @(#)netdef.h v2.17.6 tcp/ip header file, K.Furukawa, KEK, Nov.1988-Aug.1995
 *
 *	SINGLESERVER :	no unix fork facility
 *	NON_BSD :	no bsd style header files
 *	NOLINGERTIME :	no linger structure
 *	REINSTALLSIGNAL: signal reinstallation needed on systemV
 *	NOFTIME :	no ftime()
 *	NOSELECT :	no select()
 *	BADACCEPT :	accept() bug in return value
 *	NEEDTIMEVAL :	need timeval structure definition at the end
 *	NEEDFDMACRO :	need fd_xxx macros at the end
 *	DONTDEFLINT :	users don't need LInt definition
**/

#ifndef __NETDEF_H
#define __NETDEF_H

#ifdef vms				/* for dec/vms */
#define NOSELECT 1			/* no select() call */
#ifndef MULTINET			/* not multinet */
#ifndef UCX				/* not ucx */
#define NON_BSD 1			/* dec/vms-win/tcp non-BSD system */
#endif
#endif
#ifdef NON_BSD				/* for dec/vms-win/tcp */
#include stdio				/* standard buffered i/o routines */
#include time				/* time definitions */
#include <sys/types2.h>			/* general data type */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#define SINGLESERVER 1			/* no multiple server */
#define NON_BSD 1			/* non-BSD system */
#else /* NON_BSD */			/* for dec/vms-multinet or ucx */
#define SINGLESERVER 1			/* no multiple server */
#ifdef MULTINET
#define netclose(d) socket_close(d)	/* close function */
#endif
#ifdef UCX
#ifndef getservbyname
#define getservbyname(a, b) zerozero()	/* ucx 2.0/3.0 have no getservbyname */
/* void *zerozero()  { return 0; } */
#endif /* getservbyname */
#define netclose(d) close(d)		/* close function */
#define bcopy(s,d,c) memcpy(d,s,c)	/* bsd subroutines */
#define bzero(d,c) memset(d,0,c)
#define index(s,c) strchr(s,c)		/* character search */
#define rindex(s,c) strrchr(s,c)
#endif
					/* using BSD definitions at the end */
#endif
#endif

#ifdef sony				/* for sony/news */
					/* using BSD definitions at the end */
#endif

#ifdef sun				/* for smi/sun3 and sun4 */
					/* using BSD definitions at the end */
#endif

#ifdef ultrix				/* for dec/ultrix */
					/* using BSD definitions at the end */
#endif

#ifdef NeXT				/* for NeXT */
					/* using BSD definitions at the end */
#endif

#ifdef __osf__				/* for dec/alpha/osf1 */
					/* using BSD definitions at the end */
#define LINT32				/* int is 32bit and long is 64bit */
#endif

#ifdef hpux				/* for hpux */
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines, hpux 8.0- */
#include <netdb.h>			/* network get(set)xxx routines */
#define bcopy(s,d,c) memcpy(d,s,c)	/* bsd subroutines */
#define bzero(d,c) memset(d,0,c)
/* #define NOSELECT 1 */		/* no bsd select() call */
#define NOLINGERTIME 1			/* no linger structure defined */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif

#ifdef SXA				/* for Fujitsu/A60/SXA */
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <net/in.h>			/* another network structures */
#include <net/inet.h>			/* inet_xxx routines */
#include <net/netdb.h>			/* network get(set)xxx routines */
#define NOSELECT 1			/* no select() call ?? */
#define NOLINGERTIME 1			/* no linger structure defined */
#define NOFTIME 1			/* no ftime() */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif

#ifdef sgi				/* for sg/iris/irix */
#include <stdio.h>			/* standard buffered i/o routines */
#include <bsd/sys/types.h>		/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <bsd/sys/socket.h>		/* network structures */
#include <bsd/netinet/in.h>		/* another network structures */
#include <bsd/arpa/inet.h>		/* inet_xxx routines */
#include <bsd/netdb.h>			/* network get(set)xxx routines */
#define NOSELECT 1			/* no select() call ?? */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif

#ifdef _AIX				/* for ibm/aix */
					/* using BSD definitions at the end */
#include <sys/select.h>			/* for select() call */
/* #define REINSTALLSIGNAL 1 */		/* signal reinstallation */
					/*  not nessesary on aix 3.2 */
#endif

#ifdef ebcdic				/* for mitsubishi/umx */
#define  TCP_IP  1
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/time.h>			/* time definitions for select */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <sys/net/in.h>			/* another network structures */
/* #include <net/inet.h> */
unsigned long inet_addr();		/* inet_xxx routine */
char *inet_ntoa();			/* inet_xxx routine */
#include <net/netdb.h>			/* network get(set)xxx routines */
#define index(s,c) strchr(s,c)		/* character search */
#define rindex(s,c) strrchr(s,c)
/* #define NOSELECT 1 */		/* no select() call ?? */
#define NOFTIME 1			/* no ftime() */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif

#ifdef _AUX_SOURCE			/* for apple/macintosh/aux */
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <netdb.h>			/* network get(set)xxx routines */
#define bcopy(s,d,c) memcpy(d,s,c)	/* bsd subroutines */
#define bzero(d,c) memset(d,0,c)
#define NOSELECT 1			/* no select() call ?? */
#define NOFTIME 1			/* no ftime() */
#define REINSTALLSIGNAL 1		/* signal reinstallation */
#define NON_BSD 1			/* non-BSD system */
#endif

#ifdef Lynx				/* for Lynx-OS */
					/* using BSD definitions at the end */
#endif

#if (defined(_WIN32) && !defined(__CYGWIN32__) && !defined(__OPENNT))
					/* for Windows/Winsock */
					/* we should use winsock2 soon */
#define WINSOCK 1
#include <stdio.h>			/* standard buffered i/o routines */
#include <stdlib.h>			/* standard buffered i/o routines */
#include <string.h>			/* string/memory, may trouble ... */
#include <winsock.h>
#include <sys/types.h>			/* general data type */
#include <sys/timeb.h>			/* time definitions for ftime */
#include <signal.h>			/* signal names */
#include <errno.h>			/* for errno */
#ifndef NONEEDTIMEH
#include <time.h>			/* time definitions such as ctime */
#endif /* NONEEDTIMEH */
#ifdef SIGACTION
#include <sys/wait.h>			/* waitpid() */
#endif /* SIGACTION */
#ifndef SIGCLD
#define SIGCLD (SIGCHLD)		/* for bsd (cygwin32) */
#endif /* SIGCLD */
#define SINGLESERVER 1			/* no multiple server */
#define USEFTIME 1			/* use ftime instead of gettimeofday */
#define NOSELECT 1			/* no select() call in 1.3 */
#define NON_BSD 1			/* non-BSD system */
#define LITTLE_END 1			/* little endian machine */
#define sleep(t) s2_null()
#endif /* _WIN32 */

#ifdef OSK				/* for os9/68k ISP */
#include <stdio.h>			/* standard buffered i/o routines */
#include <types.h>			/* general data type */
#include <time.h>			/* time definitions */
/* #include <sys/timeb.h> */		/* timeb definitions from cabrales */
#include <inet/socket.h>		/* network structures */
#include <inet/in.h>			/* another network structures */
#include <inet/netdb.h>			/* get(set)xxx and inet_xxx routines */
#define bcopy(s,d,c) memcpy(d,s,c)	/* bsd subroutines */
#define bzero(d,c) memset(d,0,c)
#ifndef perror
extern int errno;
#define perror(str) prerr(0,errno);fprintf(stderr,"%s\n",str)
#endif /* perror */
#define SINGLESERVER 1			/* no multiple server */
#define NOSELECT 1			/* no select() call in 1.3 */
/* select() from cabrales */
/* #define NOFTIME 1 */			/* no ftime(), ?? */
/* ftime() from cabrales */
#define BADACCEPT 1			/* accept() bug */
#define NON_BSD 1			/* non-BSD system */
#define NEEDTIMEVAL 1			/* timeval structure */
#define NEEDTIMEB 1			/* timeb structure */
#define NEEDFDMACRO 1			/* fd_xxx macros */
#define EINTR 4				/* interrupted system call on unix */
#endif

#ifdef MSDOS				/* for msdos/pctcp(ftp soft. inc.) */
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
#include <time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <4bsddefs.h>			/* bsd emulation definitions */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#define index(s,c) strchr(s,c)		/* character search */
#define rindex(s,c) strrchr(s,c)
#define SINGLESERVER 1			/* no multiple server */
#define BADACCEPT 1			/* accept() bug */
#define NON_BSD 1			/* non-BSD system */
#define LINT16				/* int is 16 bit */
#endif

#ifndef NON_BSD				/* for BSD based systems */
#include <stdio.h>			/* standard buffered i/o routines */
#include <sys/types.h>			/* general data type */
/* #include <time.h>			 * time definitions */
#include <sys/time.h>			/* time definitions */
#include <sys/timeb.h>			/* time definitions */
#include <signal.h>			/* signal names */
#include <sys/socket.h>			/* network structures */
#include <netinet/in.h>			/* another network structures */
#include <arpa/inet.h>			/* inet_xxx routines */
#include <netdb.h>			/* network get(set)xxx routines */
#endif

#ifdef NEEDTIMEVAL			/* timeval structure */
struct timeval {			/*  in sys/time.h on unix */
    long	tv_sec;			/* time in seconds */
    long	tv_usec;		/* remaining time in microseconds */
};
#endif /* NEEDTIMEVAL */

#ifdef NEEDTIMEB			/* timeb structure */
struct timeb {				/*  in sys/timeb.h on unix */
    time_t	time;			/* time in seconds */
    unsigned	short millitm;		/* remaining time in milliseconds */
    short timezone;			/* time zone */
    short dstflag;			/* daylight saving time flag */
};
#endif /* NEEDTIMEB */

#ifdef NEEDFDMACRO			/* fd_xxx macros */
#define NBBY    8
#ifndef FD_SETSIZE
#define FD_SETSIZE	256
#endif
typedef long    fd_mask;
#define NFDBITS (sizeof(fd_mask) * NBBY)
#ifndef howmany
#define howmany(x, y)   (((x)+((y)-1))/(y))
#endif
typedef struct fd_set {
    fd_mask fds_bits[howmany(FD_SETSIZE, NFDBITS)];
} fd_set;
#define FD_SET(n, p)    ((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define FD_CLR(n, p)    ((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define FD_ISSET(n, p)  ((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)      memset((char *)(p), 0, sizeof(*(p)))
#endif /* NEEDFDMACRO */

#ifndef DONTDEFLINT
#ifdef LINT32
typedef int	LInt32;			/* 32bit integer */
#else  /* LINT32 */
typedef long	LInt32;
#endif  /* LINT32 */
#ifdef LINT16
typedef int	LInt16;			/* 16bit integer */
#else  /* LINT16 */
typedef short	LInt16;
#endif  /* LINT16 */
#endif  /* DONTDEFLINT */

#endif /* __NETDEF_H */

#endif  /* __STDC__ */
